@@ -11,10 +11,10 @@ more details.
 
 You should have received a copy of the GNU Lesser General Public License
 along with this library; if not, write to the Free Software Foundation, Inc.,
-59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 **********/
 // "liveMedia"
-// Copyright (c) 1996-2007 Live Networks, Inc.  All rights reserved.
+// Copyright (c) 1996-2010 Live Networks, Inc.  All rights reserved.
 // Demultiplexer for a MPEG 1 or 2 Program Stream
 // Implementation
 
@@ -30,7 +30,7 @@ enum MPEGParseState {
   PARSING_PACK_HEADER,
   PARSING_SYSTEM_HEADER,
   PARSING_PES_PACKET
-}; 
+};
 
 class MPEGProgramStreamParser: public StreamParser {
 public:
@@ -92,7 +92,7 @@ MPEG1or2Demux
     fOutput[i].isPotentiallyReadable = False;
     fOutput[i].isCurrentlyActive = False;
     fOutput[i].isCurrentlyAwaitingData = False;
-  } 
+  }
 }
 
 MPEG1or2Demux::~MPEG1or2Demux() {
@@ -157,14 +157,14 @@ void MPEG1or2Demux::registerReadInterest(u_int8_t streamIdTag,
 				     FramedSource::onCloseFunc* onCloseFunc,
 				     void* onCloseClientData) {
   struct OutputDescriptor& out = fOutput[streamIdTag];
-    
+
   // Make sure this stream is not already being read:
   if (out.isCurrentlyAwaitingData) {
     envir() << "MPEG1or2Demux::registerReadInterest(): attempt to read stream id "
 	    << (void*)streamIdTag << " more than once!\n";
-    exit(1);
+    abort();
   }
-    
+
   out.to = to; out.maxSize = maxSize;
   out.fAfterGettingFunc = afterGettingFunc;
   out.afterGettingClientData = afterGettingClientData;
@@ -229,7 +229,7 @@ void MPEG1or2Demux::continueReadProcessing() {
 
     if (acquiredStreamIdTag != 0) {
       // We were able to acquire a frame from the input.
-      struct OutputDescriptor& newOut = fOutput[acquiredStreamIdTag]; 
+      struct OutputDescriptor& newOut = fOutput[acquiredStreamIdTag];
       newOut.isCurrentlyAwaitingData = False;
         // indicates that we can be read again
         // (This needs to be set before the 'after getting' call below,
@@ -311,7 +311,7 @@ void MPEG1or2Demux::handleClosure(void* clientData) {
     out.savedDataTotalSize = 0;
     out.isPotentiallyReadable = out.isCurrentlyActive = out.isCurrentlyAwaitingData
       = False;
-  } 
+  }
   for (i = 0; i < numPending; ++i) {
     (*savedPending[i].fOnCloseFunc)(savedPending[i].onCloseClientData);
   }
@@ -411,9 +411,9 @@ void MPEGProgramStreamParser::parsePackHeader() {
       skipBytes(1);
     }
   }
-  
+
   // The size of the pack header differs depending on whether it's
-  // MPEG-1 or MPEG-2.  The next byte tells us this: 
+  // MPEG-1 or MPEG-2.  The next byte tells us this:
   unsigned char nextByte = get1Byte();
   MPEG1or2Demux::SCR& scr = fUsingSource->fLastSeenSCR; // alias
   if ((nextByte&0xF0) == 0x20) { // MPEG-1
@@ -460,10 +460,10 @@ void MPEGProgramStreamParser::parsePackHeader() {
 			  << (void*)nextByte
 			  << " following pack_start_code\n";
   }
-  
+
   // Check for a System Header next:
   setParseState(PARSING_SYSTEM_HEADER);
-}  
+}
 
 void MPEGProgramStreamParser::parseSystemHeader() {
 #ifdef DEBUG
@@ -475,14 +475,14 @@ void MPEGProgramStreamParser::parseSystemHeader() {
     setParseState(PARSING_PES_PACKET);
     return;
   }
-  
+
 #ifdef DEBUG
   fprintf(stderr, "saw system_header_start_code\n"); fflush(stderr);
 #endif
   skipBytes(4); // we've already seen the system_header_start_code
-  
+
   unsigned short remaining_header_length = get2Bytes();
-  
+
   // According to the MPEG-1 and MPEG-2 specs, "remaining_header_length" should be
   // at least 6 bytes.  Check this now:
   if (remaining_header_length < 6) {
@@ -490,7 +490,7 @@ void MPEGProgramStreamParser::parseSystemHeader() {
 			  << remaining_header_length << " < 6\n";
   }
   skipBytes(remaining_header_length);
-  
+
   // Check for a PES Packet next:
   setParseState(PARSING_PES_PACKET);
 }
@@ -517,7 +517,7 @@ Boolean MPEGProgramStreamParser
   }
 }
 
-#define READER_NOT_READY 2 
+#define READER_NOT_READY 2
 
 unsigned char MPEGProgramStreamParser::parsePESPacket() {
 #ifdef DEBUG
@@ -529,7 +529,7 @@ unsigned char MPEGProgramStreamParser::parsePESPacket() {
     setParseState(PARSING_PACK_HEADER);
     return 0;
   }
-  
+
 #ifdef DEBUG
   fprintf(stderr, "saw packet_start_code_prefix\n"); fflush(stderr);
 #endif
@@ -538,7 +538,7 @@ unsigned char MPEGProgramStreamParser::parsePESPacket() {
   unsigned char stream_id = get1Byte();
 #if defined(DEBUG) || defined(DEBUG_TIMESTAMPS)
   unsigned char streamNum = stream_id;
-  char* streamTypeStr;
+  char const* streamTypeStr;
   if ((stream_id&0xE0) == 0xC0) {
     streamTypeStr = "audio";
     streamNum = stream_id&~0xE0;
@@ -561,7 +561,7 @@ unsigned char MPEGProgramStreamParser::parsePESPacket() {
   static unsigned frameCount = 1;
   fprintf(stderr, "%d, saw %s stream: 0x%02x\n", frameCount, streamTypeStr, streamNum); fflush(stderr);
 #endif
-  
+
   unsigned short PES_packet_length = get2Bytes();
 #ifdef DEBUG
   fprintf(stderr, "PES_packet_length: %d\n", PES_packet_length); fflush(stderr);
@@ -572,7 +572,7 @@ unsigned char MPEGProgramStreamParser::parsePESPacket() {
   if (fUsingSource->fOutput[RAW_PES].isPotentiallyReadable) {
     // Hack: We've been asked to return raw PES packets, for every stream:
     stream_id = RAW_PES;
-  } 
+  }
   unsigned savedParserOffset = curOffset();
 #ifdef DEBUG_TIMESTAMPS
   unsigned char pts_highBit = 0;
@@ -691,12 +691,12 @@ unsigned char MPEGProgramStreamParser::parsePESPacket() {
   } else {
     PES_packet_length -= bytesSkipped;
 #ifdef DEBUG
-    unsigned next4Bytes = test4Bytes(); 
+    unsigned next4Bytes = test4Bytes();
 #endif
 
     // Check whether our using source is interested in this stream type.
     // If so, deliver the frame to him:
-    MPEG1or2Demux::OutputDescriptor_t& out = fUsingSource->fOutput[stream_id]; 
+    MPEG1or2Demux::OutputDescriptor_t& out = fUsingSource->fOutput[stream_id];
     if (out.isCurrentlyAwaitingData) {
       unsigned numBytesToCopy;
       if (PES_packet_length > out.maxSize) {
