@@ -11,10 +11,10 @@ more details.
 
 You should have received a copy of the GNU Lesser General Public License
 along with this library; if not, write to the Free Software Foundation, Inc.,
-59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 **********/
 // "liveMedia"
-// Copyright (c) 1996-2004 Live Networks, Inc.  All rights reserved.
+// Copyright (c) 1996-2010 Live Networks, Inc.  All rights reserved.
 // A MPEG 1 or 2 Elementary Stream, demultiplexed from a Program Stream
 // Implementation
 
@@ -56,8 +56,9 @@ char const* MPEG1or2DemuxedElementaryStream::MIMEtype() const {
 }
 
 unsigned MPEG1or2DemuxedElementaryStream::maxFrameSize() const {
-  return 65536;
-      // because the MPEG spec allows for PES packets as large as ~65536 bytes
+  return 6+65535;
+      // because the MPEG spec allows for PES packets as large as
+      // (6 + 65535) bytes (header + data)
 }
 
 void MPEG1or2DemuxedElementaryStream
