@@ -11,10 +11,10 @@ more details.
 
 You should have received a copy of the GNU Lesser General Public License
 along with this library; if not, write to the Free Software Foundation, Inc.,
-59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 **********/
 // "liveMedia"
-// Copyright (c) 1996-2007 Live Networks, Inc.  All rights reserved.
+// Copyright (c) 1996-2010 Live Networks, Inc.  All rights reserved.
 // A filter that passes through (unchanged) chunks that contain an integral number
 // of MPEG-2 Transport Stream packets, but returning (in "fDurationInMicroseconds")
 // an updated estimate of the time gap between chunks.
@@ -32,6 +32,7 @@ along with this library; if not, write to the Free Software Foundation, Inc.,
   // times that we expect receivers to play the incoming packets).
   // (must be <= 1)
 #define MAX_PLAYOUT_BUFFER_DURATION 0.1 // (seconds)
+#define PCR_PERIOD_VARIATION_RATIO 0.5
 
 ////////// PIDStatus //////////
 
@@ -58,16 +59,20 @@ MPEG2TransportStreamFramer* MPEG2TransportStreamFramer
 MPEG2TransportStreamFramer
 ::MPEG2TransportStreamFramer(UsageEnvironment& env, FramedSource* inputSource)
   : FramedFilter(env, inputSource),
-    fTSPacketCount(0), fTSPacketDurationEstimate(0.0) {
+    fTSPacketCount(0), fTSPacketDurationEstimate(0.0), fTSPCRCount(0) {
   fPIDStatusTable = HashTable::create(ONE_WORD_HASH_KEYS);
 }
 
 MPEG2TransportStreamFramer::~MPEG2TransportStreamFramer() {
+  clearPIDStatusTable();
+  delete fPIDStatusTable;
+}
+
+void MPEG2TransportStreamFramer::clearPIDStatusTable() {
   PIDStatus* pidStatus;
   while ((pidStatus = (PIDStatus*)fPIDStatusTable->RemoveNext()) != NULL) {
     delete pidStatus;
   }
-  delete fPIDStatusTable;
 }
 
 void MPEG2TransportStreamFramer::doGetNextFrame() {
@@ -81,12 +86,9 @@ void MPEG2TransportStreamFramer::doGetNextFrame() {
 void MPEG2TransportStreamFramer::doStopGettingFrames() {
   FramedFilter::doStopGettingFrames();
   fTSPacketCount = 0;
+  fTSPCRCount = 0;
 
-  // Clear out the existing PID status table:
-  PIDStatus* pidStatus;
-  while ((pidStatus = (PIDStatus*)fPIDStatusTable->RemoveNext()) != NULL) {
-    delete pidStatus;
-  }
+  clearPIDStatusTable();
 }
 
 void MPEG2TransportStreamFramer
@@ -172,6 +174,7 @@ void MPEG2TransportStreamFramer
   if (pcrFlag == 0) return; // no PCR
 
   // There's a PCR.  Get it, and the PID:
+  ++fTSPCRCount;
   u_int32_t pcrBaseHigh = (pkt[6]<<24)|(pkt[7]<<16)|(pkt[8]<<8)|pkt[9];
   double clock = pcrBaseHigh/45000.0;
   if ((pkt[10]&0x80) != 0) clock += 1/90000.0; // add in low-bit (if set)
@@ -182,6 +185,7 @@ void MPEG2TransportStreamFramer
 
   // Check whether we already have a record of a PCR for this PID:
   PIDStatus* pidStatus = (PIDStatus*)(fPIDStatusTable->Lookup((char*)pid));
+
   if (pidStatus == NULL) {
     // We're seeing this PID's PCR for the first time:
     pidStatus = new PIDStatus(clock, timeNow);
@@ -193,6 +197,15 @@ void MPEG2TransportStreamFramer
     // We've seen this PID's PCR before; update our per-packet duration estimate:
     double durationPerPacket
       = (clock - pidStatus->lastClock)/(fTSPacketCount - pidStatus->lastPacketNum);
+
+    // Hack (suggested by "Romain"): Don't update our estimate if this PCR appeared unusually quickly.
+    // (This can produce more accurate estimates for wildly VBR streams.)
+    double meanPCRPeriod = 0.0;
+    if (fTSPCRCount > 0) {
+      meanPCRPeriod=(double)fTSPacketCount/fTSPCRCount;
+      if (fTSPacketCount - pidStatus->lastPacketNum < meanPCRPeriod*PCR_PERIOD_VARIATION_RATIO) return;
+    }
+
     if (fTSPacketDurationEstimate == 0.0) { // we've just started
       fTSPacketDurationEstimate = durationPerPacket;
     } else if (discontinuity_indicator == 0 && durationPerPacket >= 0.0) {
@@ -216,7 +229,7 @@ void MPEG2TransportStreamFramer
       pidStatus->firstRealTime = timeNow;
     }
 #ifdef DEBUG_PCR
-    fprintf(stderr, "PID 0x%x, PCR 0x%08x+%d:%03x == %f @ %f (diffs %f @ %f), pkt #%lu, discon %d => this duration %f, new estimate %f\n", pid, pcrBaseHigh, pkt[10]>>7, pcrExt, clock, timeNow, clock - pidStatus->firstClock, timeNow - pidStatus->firstRealTime, fTSPacketCount, discontinuity_indicator != 0, durationPerPacket, fTSPacketDurationEstimate);
+    fprintf(stderr, "PID 0x%x, PCR 0x%08x+%d:%03x == %f @ %f (diffs %f @ %f), pkt #%lu, discon %d => this duration %f, new estimate %f, mean PCR period=%f\n", pid, pcrBaseHigh, pkt[10]>>7, pcrExt, clock, timeNow, clock - pidStatus->firstClock, timeNow - pidStatus->firstRealTime, fTSPacketCount, discontinuity_indicator != 0, durationPerPacket, fTSPacketDurationEstimate, meanPCRPeriod );
 #endif
   }
 
