@@ -11,10 +11,10 @@ more details.
 
 You should have received a copy of the GNU Lesser General Public License
 along with this library; if not, write to the Free Software Foundation, Inc.,
-59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 **********/
 // "liveMedia"
-// Copyright (c) 1996-2004 Live Networks, Inc.  All rights reserved.
+// Copyright (c) 1996-2010 Live Networks, Inc.  All rights reserved.
 // RTP source for a common kind of payload format: Those that pack multiple,
 // complete codec frames (as many as possible) into each RTP packet.
 // Implementation
@@ -29,14 +29,20 @@ class ReorderingPacketBuffer {
 public:
   ReorderingPacketBuffer(BufferedPacketFactory* packetFactory);
   virtual ~ReorderingPacketBuffer();
+  void reset();
 
   BufferedPacket* getFreePacket(MultiFramedRTPSource* ourSource);
-  void storePacket(BufferedPacket* bPacket);
+  Boolean storePacket(BufferedPacket* bPacket);
   BufferedPacket* getNextCompletedPacket(Boolean& packetLossPreceded);
   void releaseUsedPacket(BufferedPacket* packet);
   void freePacket(BufferedPacket* packet) {
-    if (packet != fSavedPacket) delete packet;
+    if (packet != fSavedPacket) {
+      delete packet;
+    } else {
+      fSavedPacketFree = True;
+    }
   }
+  Boolean isEmpty() const { return fHeadPacket == NULL; }
 
   void setThresholdTime(unsigned uSeconds) { fThresholdTime = uSeconds; }
 
@@ -48,6 +54,7 @@ private:
   BufferedPacket* fHeadPacket;
   BufferedPacket* fSavedPacket;
       // to avoid calling new/free in the common case
+  Boolean fSavedPacketFree;
 };
 
 
@@ -58,17 +65,22 @@ MultiFramedRTPSource
 		       unsigned char rtpPayloadFormat,
 		       unsigned rtpTimestampFrequency,
 		       BufferedPacketFactory* packetFactory)
-  : RTPSource(env, RTPgs, rtpPayloadFormat, rtpTimestampFrequency),
-    fCurrentPacketBeginsFrame(True/*by default*/),
-    fCurrentPacketCompletesFrame(True/*by default*/),
-    fAreDoingNetworkReads(False), fNeedDelivery(False),
-    fPacketLossInFragmentedFrame(False) {
+  : RTPSource(env, RTPgs, rtpPayloadFormat, rtpTimestampFrequency) {
+  reset();
   fReorderingBuffer = new ReorderingPacketBuffer(packetFactory);
 
   // Try to use a big receive buffer for RTP:
   increaseReceiveBufferTo(env, RTPgs->socketNum(), 50*1024);
 }
 
+void MultiFramedRTPSource::reset() {
+  fCurrentPacketBeginsFrame = True; // by default
+  fCurrentPacketCompletesFrame = True; // by default
+  fAreDoingNetworkReads = False;
+  fNeedDelivery = False;
+  fPacketLossInFragmentedFrame = False;
+}
+
 MultiFramedRTPSource::~MultiFramedRTPSource() {
   fRTPInterface.stopNetworkReading();
   delete fReorderingBuffer;
@@ -89,6 +101,12 @@ Boolean MultiFramedRTPSource
   return True;
 }
 
+void MultiFramedRTPSource::doStopGettingFrames() {
+  fRTPInterface.stopNetworkReading();
+  fReorderingBuffer->reset();
+  reset();
+}
+
 void MultiFramedRTPSource::doGetNextFrame() {
   if (!fAreDoingNetworkReads) {
     // Turn on background read handling of incoming packets:
@@ -156,7 +174,7 @@ void MultiFramedRTPSource::doGetNextFrame1() {
 		    fPresentationTime, fCurPacketHasBeenSynchronizedUsingRTCP,
 		    fCurPacketMarkerBit);
     fFrameSize += frameSize;
-    
+
     if (!nextPacket->hasUsableData()) {
       // We're completely done with this packet now
       fReorderingBuffer->releaseUsedPacket(nextPacket);
@@ -169,10 +187,17 @@ void MultiFramedRTPSource::doGetNextFrame1() {
 		<< fSavedMaxSize << ").  "
 		<< fNumTruncatedBytes << " bytes of trailing data will be dropped!\n";
       }
-      // Call our own 'after getting' function.  Because we're preceded
-      // by a network read, we can call this directly, without risking
-      // infinite recursion.
-      afterGetting(this);
+      // Call our own 'after getting' function, so that the downstream object can consume the data:
+      if (fReorderingBuffer->isEmpty()) {
+	// Common case optimization: There are no more queued incoming packets, so this code will not get
+	// executed again without having first returned to the event loop.  Call our 'after getting' function
+	// directly, because there's no risk of a long chain of recursion (and thus stack overflow):
+	afterGetting(this);
+      } else {
+	// Special case: Call our 'after getting' function via the event loop.
+	nextTask() = envir().taskScheduler().scheduleDelayedTask(0,
+								 (TaskFunc*)FramedSource::afterGetting, this);
+      }
     } else {
       // This packet contained fragmented data, and does not complete
       // the data that the client wants.  Keep getting data:
@@ -204,22 +229,22 @@ void MultiFramedRTPSource::networkReadHandler(MultiFramedRTPSource* source,
        // don't wait for 'lost' packets to arrive out-of-order later
     if ((our_random()%10) == 0) break; // simulate 10% packet loss
 #endif
-    
+
     // Check for the 12-byte RTP header:
     if (bPacket->dataSize() < 12) break;
     unsigned rtpHdr = ntohl(*(unsigned*)(bPacket->data())); ADVANCE(4);
     Boolean rtpMarkerBit = (rtpHdr&0x00800000) >> 23;
     unsigned rtpTimestamp = ntohl(*(unsigned*)(bPacket->data()));ADVANCE(4);
     unsigned rtpSSRC = ntohl(*(unsigned*)(bPacket->data())); ADVANCE(4);
-    
+
     // Check the RTP version number (it should be 2):
     if ((rtpHdr&0xC0000000) != 0x80000000) break;
-    
+
     // Skip over any CSRC identifiers in the header:
     unsigned cc = (rtpHdr>>24)&0xF;
     if (bPacket->dataSize() < cc) break;
     ADVANCE(cc*4);
-    
+
     // Check for (& ignore) any RTP header extension
     if (rtpHdr&0x10000000) {
       if (bPacket->dataSize() < 4) break;
@@ -228,7 +253,7 @@ void MultiFramedRTPSource::networkReadHandler(MultiFramedRTPSource* source,
       if (bPacket->dataSize() < remExtSize) break;
       ADVANCE(remExtSize);
     }
-    
+
     // Discard any padding bytes:
     if (rtpHdr&0x20000000) {
       if (bPacket->dataSize() == 0) break;
@@ -236,13 +261,13 @@ void MultiFramedRTPSource::networkReadHandler(MultiFramedRTPSource* source,
 	= (unsigned)(bPacket->data())[bPacket->dataSize()-1];
       if (bPacket->dataSize() < numPaddingBytes) break;
       bPacket->removePadding(numPaddingBytes);
-    }    
+    }
     // Check the Payload Type.
     if ((unsigned char)((rtpHdr&0x007F0000)>>16)
 	!= source->rtpPayloadFormat()) {
       break;
     }
-    
+
     // The rest of the packet is the usable data.  Record and save it:
     source->fLastReceivedSSRC = rtpSSRC;
     unsigned short rtpSeqNo = (unsigned short)(rtpHdr&0xFFFF);
@@ -256,14 +281,14 @@ void MultiFramedRTPSource::networkReadHandler(MultiFramedRTPSource* source,
 			  source->timestampFrequency(),
 			  usableInJitterCalculation, presentationTime,
 			  hasBeenSyncedUsingRTCP, bPacket->dataSize());
-  
+
     // Fill in the rest of the packet descriptor, and store it:
     struct timeval timeNow;
     gettimeofday(&timeNow, NULL);
     bPacket->assignMiscParams(rtpSeqNo, rtpTimestamp, presentationTime,
 			      hasBeenSyncedUsingRTCP, rtpMarkerBit,
 			      timeNow);
-    source->fReorderingBuffer->storePacket(bPacket);
+    if (!source->fReorderingBuffer->storePacket(bPacket)) break;
 
     readSuccess = True;
   } while (0);
@@ -292,6 +317,7 @@ BufferedPacket::~BufferedPacket() {
 void BufferedPacket::reset() {
   fHead = fTail = 0;
   fUseCount = 0;
+  fIsFirstPacket = False; // by default
 }
 
 // The following function has been deprecated:
@@ -311,11 +337,11 @@ void BufferedPacket
   // of more than one frame, on the assumption that the client doesn't
   // care.  (This is more efficient than delivering a frame at a time)
 
-  // For backwards-compatibilty with existing uses of (the now deprecated)
+  // For backwards-compatibility with existing uses of (the now deprecated)
   // "nextEnclosedFrameSize()", call that function to implement this one:
   frameSize = nextEnclosedFrameSize(framePtr, dataSize);
 
-  frameDurationInMicroseconds = 0; // by default.  Subclasses should correct this.  
+  frameDurationInMicroseconds = 0; // by default.  Subclasses should correct this.
 }
 
 Boolean BufferedPacket::fillInData(RTPInterface& rtpInterface) {
@@ -356,7 +382,7 @@ void BufferedPacket::removePadding(unsigned numBytes) {
 
 void BufferedPacket::appendData(unsigned char* newData, unsigned numBytes) {
   if (numBytes > fPacketSize-fTail) numBytes = fPacketSize - fTail;
-  memmove(&fBuf[fTail], newData, numBytes); 
+  memmove(&fBuf[fTail], newData, numBytes);
   fTail += numBytes;
 }
 
@@ -397,61 +423,70 @@ void BufferedPacket::use(unsigned char* to, unsigned toSize,
   }
 }
 
+BufferedPacketFactory::BufferedPacketFactory() {
+}
+
+BufferedPacketFactory::~BufferedPacketFactory() {
+}
+
 BufferedPacket* BufferedPacketFactory
 ::createNewPacket(MultiFramedRTPSource* /*ourSource*/) {
   return new BufferedPacket;
 }
 
 
-////////// ReorderingPacketBuffer definition //////////
+////////// ReorderingPacketBuffer implementation //////////
 
 ReorderingPacketBuffer
 ::ReorderingPacketBuffer(BufferedPacketFactory* packetFactory)
   : fThresholdTime(100000) /* default reordering threshold: 100 ms */,
-    fHaveSeenFirstPacket(False), fHeadPacket(NULL), fSavedPacket(NULL) {
+    fHaveSeenFirstPacket(False), fHeadPacket(NULL), fSavedPacket(NULL), fSavedPacketFree(True) {
   fPacketFactory = (packetFactory == NULL)
     ? (new BufferedPacketFactory)
     : packetFactory;
 }
 
 ReorderingPacketBuffer::~ReorderingPacketBuffer() {
-  if (fHeadPacket == NULL) {
-    delete fSavedPacket;
-  } else {
-    delete fHeadPacket; // will also delete fSavedPacket, because it's on the list
-  }
+  reset();
   delete fPacketFactory;
 }
 
+void ReorderingPacketBuffer::reset() {
+  if (fSavedPacketFree) delete fSavedPacket; // because fSavedPacket is not in the list
+  delete fHeadPacket; // will also delete fSavedPacket if it's in the list
+  fHaveSeenFirstPacket = False;
+  fHeadPacket = NULL;
+  fSavedPacket = NULL;
+}
+
 BufferedPacket* ReorderingPacketBuffer
 ::getFreePacket(MultiFramedRTPSource* ourSource) {
   if (fSavedPacket == NULL) { // we're being called for the first time
     fSavedPacket = fPacketFactory->createNewPacket(ourSource);
+    fSavedPacketFree = True;
   }
 
-  return fHeadPacket == NULL
-    ? fSavedPacket
-    : fPacketFactory->createNewPacket(ourSource);
+  if (fSavedPacketFree == True) {
+    fSavedPacketFree = False;
+    return fSavedPacket;
+  } else {
+    return fPacketFactory->createNewPacket(ourSource);
+  }
 }
 
-void ReorderingPacketBuffer::storePacket(BufferedPacket* bPacket) {
+Boolean ReorderingPacketBuffer::storePacket(BufferedPacket* bPacket) {
   unsigned short rtpSeqNo = bPacket->rtpSeqNo();
 
   if (!fHaveSeenFirstPacket) {
     fNextExpectedSeqNo = rtpSeqNo; // initialization
+    bPacket->isFirstPacket() = True;
     fHaveSeenFirstPacket = True;
   }
 
   // Ignore this packet if its sequence number is less than the one
   // that we're looking for (in this case, it's been excessively delayed).
-  // (But (sanity check) if the new packet's sequence number is a *lot*
-  // less, then accept it anyway.)
-  unsigned short const seqNoThreshold = 100;
-  if (seqNumLT(rtpSeqNo, fNextExpectedSeqNo)
-      && seqNumLT(fNextExpectedSeqNo, rtpSeqNo+seqNoThreshold)) {
-    return;
-  }
-  
+  if (seqNumLT(rtpSeqNo, fNextExpectedSeqNo)) return False;
+
   // Figure out where the new packet will be stored in the queue:
   BufferedPacket* beforePtr = NULL;
   BufferedPacket* afterPtr = fHeadPacket;
@@ -459,13 +494,13 @@ void ReorderingPacketBuffer::storePacket(BufferedPacket* bPacket) {
     if (seqNumLT(rtpSeqNo, afterPtr->rtpSeqNo())) break; // it comes here
     if (rtpSeqNo == afterPtr->rtpSeqNo()) {
       // This is a duplicate packet - ignore it
-      return;
+      return False;
     }
-    
+
     beforePtr = afterPtr;
     afterPtr = afterPtr->nextPacket();
   }
-  
+
   // Link our new packet between "beforePtr" and "afterPtr":
   bPacket->nextPacket() = afterPtr;
   if (beforePtr == NULL) {
@@ -473,6 +508,8 @@ void ReorderingPacketBuffer::storePacket(BufferedPacket* bPacket) {
   } else {
     beforePtr->nextPacket() = bPacket;
   }
+
+  return True;
 }
 
 void ReorderingPacketBuffer::releaseUsedPacket(BufferedPacket* packet) {
@@ -494,7 +531,8 @@ BufferedPacket* ReorderingPacketBuffer
   // of the queue:
   // ASSERT: fHeadPacket->rtpSeqNo() >= fNextExpectedSeqNo
   if (fHeadPacket->rtpSeqNo() == fNextExpectedSeqNo) {
-    packetLossPreceded = False;
+    packetLossPreceded = fHeadPacket->isFirstPacket();
+        // (The very first packet is treated as if there was packet loss beforehand.)
     return fHeadPacket;
   }
 
