@@ -11,9 +11,9 @@ more details.
 
 You should have received a copy of the GNU Lesser General Public License
 along with this library; if not, write to the Free Software Foundation, Inc.,
-59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 **********/
-// Copyright (c) 1996-2007, Live Networks, Inc.  All rights reserved
+// Copyright (c) 1996-2010, Live Networks, Inc.  All rights reserved
 // A RTSP client test program that opens a RTSP URL argument,
 // and extracts the data from each incoming RTP stream.
 
@@ -45,8 +45,6 @@ char* getSDPDescriptionFromURL(Medium* client, char const* url,
     result = rtspClient->describeURL(url);
   }
 
-  extern unsigned statusCode;
-  statusCode = rtspClient->describeStatus();
   return result;
 }
 
@@ -60,9 +58,19 @@ Boolean clientSetupSubsession(Medium* client, MediaSubsession* subsession,
 
 Boolean clientStartPlayingSession(Medium* client,
 				  MediaSession* session) {
+  extern double initialSeekTime, duration, scale;
+  double endTime = initialSeekTime;
+  if (scale > 0) {
+    if (duration <= 0) endTime = -1.0f;
+    else endTime = initialSeekTime + duration;
+  } else {
+    endTime = initialSeekTime - duration;
+    if (endTime < 0) endTime = 0.0f;
+  }
+
   if (client == NULL || session == NULL) return False;
   RTSPClient* rtspClient = (RTSPClient*)client;
-  return rtspClient->playMediaSession(*session);
+  return rtspClient->playMediaSession(*session, initialSeekTime, endTime, (float)scale);
 }
 
 Boolean clientTearDownSession(Medium* client,
