@@ -11,18 +11,15 @@ more details.
 
 You should have received a copy of the GNU Lesser General Public License
 along with this library; if not, write to the Free Software Foundation, Inc.,
-59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 **********/
-// Copyright (c) 1996-2007, Live Networks, Inc.  All rights reserved
+// Copyright (c) 1996-2010, Live Networks, Inc.  All rights reserved
 // A common framework, used for the "openRTSP" and "playSIP" applications
 // Implementation
 
 #include "playCommon.hh"
 #include "BasicUsageEnvironment.hh"
 #include "GroupsockHelper.hh"
-#ifdef SUPPORT_REAL_RTSP
-#include "../RealRTSP/include/RealRTSP.hh"
-#endif
 
 #if defined(__WIN32__) || defined(_WIN32)
 #define snprintf _snprintf
@@ -63,9 +60,11 @@ AVIFileSink* aviOut = NULL;
 Boolean audioOnly = False;
 Boolean videoOnly = False;
 char const* singleMedium = NULL;
-int verbosityLevel = 0;
-double endTime = 0;
-double endTimeSlop = -1.0; // extra seconds to play at the end
+int verbosityLevel = 1; // by default, print verbose output
+double duration = 0;
+double durationSlop = -1.0; // extra seconds to play at the end
+double initialSeekTime = 0.0f;
+double scale = 1.0f;
 unsigned interPacketGapMaxTime = 0;
 unsigned totNumPacketsReceived = ~0; // used if checking inter-packet gaps
 Boolean playContinuously = False;
@@ -88,24 +87,24 @@ unsigned short movieHeight = 180; // default
 Boolean movieHeightOptionSet = False;
 unsigned movieFPS = 15; // default
 Boolean movieFPSOptionSet = False;
-char* fileNamePrefix = "";
-unsigned fileSinkBufferSize = 20000;
+char const* fileNamePrefix = "";
+unsigned fileSinkBufferSize = 100000;
 unsigned socketInputBufferSize = 0;
 Boolean packetLossCompensate = False;
 Boolean syncStreams = False;
 Boolean generateHintTracks = False;
 unsigned qosMeasurementIntervalMS = 0; // 0 means: Don't output QOS data
-unsigned statusCode = 0;
 
 struct timeval startTime;
 
 void usage() {
   *env << "Usage: " << progName
-       << " [-p <startPortNum>] [-r|-q|-4|-i] [-a|-v] [-V] [-e <endTime>] [-E <max-inter-packet-gap-time> [-c] [-s <offset>] [-n] [-O]"
+       << " [-p <startPortNum>] [-r|-q|-4|-i] [-a|-v] [-V] [-d <duration>] [-D <max-inter-packet-gap-time> [-c] [-S <offset>] [-n] [-O]"
 	   << (controlConnectionUsesTCP ? " [-t|-T <http-port>]" : "")
        << " [-u <username> <password>"
 	   << (allowProxyServers ? " [<proxy-server> [<proxy-server-port>]]" : "")
-       << "]" << (supportCodecSelection ? " [-A <audio-codec-rtp-payload-format-code>|-D <mime-subtype-name>]" : "")
+       << "]" << (supportCodecSelection ? " [-A <audio-codec-rtp-payload-format-code>|-M <mime-subtype-name>]" : "")
+       << " [-s <initial-seek-time>] [-z <scale>]"
        << " [-w <width> -h <height>] [-f <frames-per-second>] [-y] [-H] [-Q [<measurement-interval>]] [-F <filename-prefix>] [-b <file-sink-buffer-size>] [-B <input-socket-buffer-size>] [-I <input-interface-ip-address>] [-m] <url> (or " << progName << " -o [-V] <url>)\n";
   //##### Add "-R <dest-rtsp-url>" #####
   shutdown();
@@ -170,7 +169,7 @@ int main(int argc, char** argv) {
       break;
     }
 
-    case 'I': { // specify input interface... 
+    case 'I': { // specify input interface...
       NetAddressList addresses(argv[2]);
       if (addresses.numAddresses() == 0) {
 	*env << "Failed to find network address for \"" << argv[2] << "\"";
@@ -193,29 +192,29 @@ int main(int argc, char** argv) {
       break;
     }
 
-    case 'V': { // verbose output
-      verbosityLevel = 1;
+    case 'V': { // disable verbose output
+      verbosityLevel = 0;
       break;
     }
 
-    case 'e': { // specify end time, or how much to delay after end time
+    case 'd': { // specify duration, or how much to delay after end time
       float arg;
       if (sscanf(argv[2], "%g", &arg) != 1) {
 	usage();
       }
       if (argv[2][0] == '-') { // not "arg<0", in case argv[2] was "-0"
-	// a 'negative' argument was specified; use this for "endTimeSlop":
-	endTime = 0; // use whatever's in the SDP
-	endTimeSlop = -arg;
+	// a 'negative' argument was specified; use this for "durationSlop":
+	duration = 0; // use whatever's in the SDP
+	durationSlop = -arg;
       } else {
-	endTime = arg;
-	endTimeSlop = 0;
+	duration = arg;
+	durationSlop = 0;
       }
       ++argv; --argc;
       break;
     }
 
-    case 'E': { // specify maximum number of seconds to wait for packets:
+    case 'D': { // specify maximum number of seconds to wait for packets:
       if (sscanf(argv[2], "%u", &interPacketGapMaxTime) != 1) {
 	usage();
       }
@@ -228,12 +227,12 @@ int main(int argc, char** argv) {
       break;
     }
 
-    case 's': { // specify an offset to use with "SimpleRTPSource"s
+    case 'S': { // specify an offset to use with "SimpleRTPSource"s
       if (sscanf(argv[2], "%d", &simpleRTPoffsetArg) != 1) {
 	usage();
       }
       if (simpleRTPoffsetArg < 0) {
-	*env << "offset argument to \"-s\" must be >= 0\n";
+	*env << "offset argument to \"-S\" must be >= 0\n";
 	usage();
       }
       ++argv; --argc;
@@ -319,7 +318,7 @@ int main(int argc, char** argv) {
       break;
     }
 
-    case 'D': { // specify a MIME subtype for a dynamic RTP payload type
+    case 'M': { // specify a MIME subtype for a dynamic RTP payload type
       mimeSubtype = argv[2];
       if (desiredAudioRTPPayloadFormat==0) desiredAudioRTPPayloadFormat =96;
       ++argv; --argc;
@@ -406,6 +405,26 @@ int main(int argc, char** argv) {
       break;
     }
 
+    case 's': { // specify initial seek time (trick play)
+      double arg;
+      if (sscanf(argv[2], "%lg", &arg) != 1 || arg < 0) {
+	usage();
+      }
+      initialSeekTime = arg;
+      ++argv; --argc;
+      break;
+    }
+
+    case 'z': { // scale (trick play)
+      float arg;
+      if (sscanf(argv[2], "%g", &arg) != 1 || arg == 0.0f) {
+	usage();
+      }
+      scale = arg;
+      ++argv; --argc;
+      break;
+    }
+
     default: {
       usage();
       break;
@@ -455,11 +474,11 @@ int main(int argc, char** argv) {
   if (!createReceivers && notifyOnPacketArrival) {
     *env << "Warning: Because we're not receiving stream data, the -n flag has no effect\n";
   }
-  if (endTimeSlop < 0) {
+  if (durationSlop < 0) {
     // This parameter wasn't set, so use a default value.
     // If we're measuring QOS stats, then don't add any slop, to avoid
     // having 'empty' measurement intervals at the end.
-    endTimeSlop = qosMeasurementIntervalMS > 0 ? 0.0 : 5.0;
+    durationSlop = qosMeasurementIntervalMS > 0 ? 0.0 : 5.0;
   }
 
   char* url = argv[1];
@@ -542,39 +561,42 @@ int main(int argc, char** argv) {
 
     if (createReceivers) {
       if (!subsession->initiate(simpleRTPoffsetArg)) {
-		*env << "Unable to create receiver for \"" << subsession->mediumName()
-			<< "/" << subsession->codecName()
-			<< "\" subsession: " << env->getResultMsg() << "\n";
+	*env << "Unable to create receiver for \"" << subsession->mediumName()
+	     << "/" << subsession->codecName()
+	     << "\" subsession: " << env->getResultMsg() << "\n";
       } else {
-		*env << "Created receiver for \"" << subsession->mediumName()
-			<< "/" << subsession->codecName()
-			<< "\" subsession (client ports " << subsession->clientPortNum()
-			<< "-" << subsession->clientPortNum()+1 << ")\n";
-		madeProgress = True;
-
-		if (subsession->rtpSource() != NULL) {
-		  // Because we're saving the incoming data, rather than playing
-		  // it in real time, allow an especially large time threshold
-		  // (1 second) for reordering misordered incoming packets:
-		  unsigned const thresh = 1000000; // 1 second 
-		  subsession->rtpSource()->setPacketReorderingThresholdTime(thresh);
-
-		  if (socketInputBufferSize > 0) {
-		    // Set the RTP source's input buffer size as specified:
-		    int socketNum
-		      = subsession->rtpSource()->RTPgs()->socketNum();
-		    unsigned curBufferSize
-		      = getReceiveBufferSize(*env, socketNum);
-		    unsigned newBufferSize
-		      = setReceiveBufferTo(*env, socketNum, socketInputBufferSize);
-		    *env << "Changed socket receive buffer size for the \""
-			 << subsession->mediumName()
-			 << "/" << subsession->codecName()
-			 << "\" subsession from "
-			 << curBufferSize << " to "
-			 << newBufferSize << " bytes\n";
-		  }
-		}
+	*env << "Created receiver for \"" << subsession->mediumName()
+	     << "/" << subsession->codecName()
+	     << "\" subsession (client ports " << subsession->clientPortNum()
+	     << "-" << subsession->clientPortNum()+1 << ")\n";
+	madeProgress = True;
+	
+	if (subsession->rtpSource() != NULL) {
+	  // Because we're saving the incoming data, rather than playing
+	  // it in real time, allow an especially large time threshold
+	  // (1 second) for reordering misordered incoming packets:
+	  unsigned const thresh = 1000000; // 1 second
+	  subsession->rtpSource()->setPacketReorderingThresholdTime(thresh);
+	  
+	  // Set the RTP source's OS socket buffer size as appropriate - either if we were explicitly asked (using -B),
+	  // or if the desired FileSink buffer size happens to be larger than the current OS socket buffer size.
+	  // (The latter case is a heuristic, on the assumption that if the user asked for a large FileSink buffer size,
+	  // then the input data rate may be large enough to justify increasing the OS socket buffer size also.)
+	  int socketNum = subsession->rtpSource()->RTPgs()->socketNum();
+	  unsigned curBufferSize = getReceiveBufferSize(*env, socketNum);
+	  if (socketInputBufferSize > 0 || fileSinkBufferSize > curBufferSize) {
+	    unsigned newBufferSize = socketInputBufferSize > 0 ? socketInputBufferSize : fileSinkBufferSize;
+	    newBufferSize = setReceiveBufferTo(*env, socketNum, newBufferSize);
+	    if (socketInputBufferSize > 0) { // The user explicitly asked for the new socket buffer size; announce it:
+	      *env << "Changed socket receive buffer size for the \""
+		   << subsession->mediumName()
+		   << "/" << subsession->codecName()
+		   << "\" subsession from "
+		   << curBufferSize << " to "
+		   << newBufferSize << " bytes\n";
+	    }
+	  }
+	}
       }
     } else {
       if (subsession->clientPortNum() == 0) {
@@ -582,7 +604,7 @@ int main(int argc, char** argv) {
 	     << subsession->mediumName()
 	     << "/" << subsession->codecName()
 	     << "\" subsession.  (Try adding the \"-p <portNum>\" option.)\n";
-      } else {	
+      } else {
 		madeProgress = True;
       }
     }
@@ -623,52 +645,13 @@ int main(int argc, char** argv) {
       }
 
       aviOut->startPlaying(sessionAfterPlaying, NULL);
-#ifdef SUPPORT_REAL_RTSP
-    } else if (session->isRealNetworksRDT) {
-      // For RealNetworks' sessions, we create a single output file,
-      // named "output.rm".
-      char outFileName[1000];
-      if (singleMedium == NULL) {
-	snprintf(outFileName, sizeof outFileName, "%soutput.rm", fileNamePrefix);
-      } else {
-	// output to 'stdout' as normal, even though we actually output all media
-	sprintf(outFileName, "stdout");
-      }
-      FileSink* fileSink = FileSink::createNew(*env, outFileName,
-					       fileSinkBufferSize, oneFilePerFrame);
-
-      // The output file needs to begin with a special 'RMFF' header,
-      // in order for it to be usable.  Write this header first:
-      unsigned headerSize;
-      unsigned char* headerData = RealGenerateRMFFHeader(session, headerSize);
-      struct timeval timeNow;
-      gettimeofday(&timeNow, NULL);
-      fileSink->addData(headerData, headerSize, timeNow);
-      delete[] headerData;
-
-      // Start playing the output file from the first subsession.
-      // (Hack: Because all subsessions' data is actually multiplexed on the
-      // single RTSP TCP connection, playing from one subsession is sufficient.)
-      iter.reset();
-      madeProgress = False;
-      while ((subsession = iter.next()) != NULL) {
-	if (subsession->readSource() == NULL) continue; // was not initiated
-
-	  fileSink->startPlaying(*(subsession->readSource()),
-					 subsessionAfterPlaying,
-					 subsession);
-	  madeProgress = True;
-	  break; // play from one subsession only
-      }
-      if (!madeProgress) shutdown();
-#endif
     } else {
       // Create and start "FileSink"s for each subsession:
       madeProgress = False;
       iter.reset();
       while ((subsession = iter.next()) != NULL) {
 	if (subsession->readSource() == NULL) continue; // was not initiated
-	
+
 	// Create an output file for each desired stream:
 	char outFileName[1000];
 	if (singleMedium == NULL) {
@@ -690,7 +673,7 @@ int main(int argc, char** argv) {
 						 fileSinkBufferSize, oneFilePerFrame);
 	} else if (strcmp(subsession->mediumName(), "video") == 0 &&
 	    (strcmp(subsession->codecName(), "H264") == 0)) {
-      // For H.264 video stream, we use a special sink that insert start_codes:
+	  // For H.264 video stream, we use a special sink that insert start_codes:
 	  fileSink = H264VideoFileSink::createNew(*env, outFileName,
 						 fileSinkBufferSize, oneFilePerFrame);
 	} else {
@@ -729,7 +712,7 @@ int main(int argc, char** argv) {
 	  subsession->sink->startPlaying(*(subsession->readSource()),
 					 subsessionAfterPlaying,
 					 subsession);
-	  
+
 	  // Also set a handler to be called if a RTCP "BYE" arrives
 	  // for this subsession:
 	  if (subsession->rtcpInstance() != NULL) {
@@ -743,7 +726,7 @@ int main(int argc, char** argv) {
       if (!madeProgress) shutdown();
     }
   }
-    
+
   // Finally, start playing each subsession, to start the data flow:
 
   startPlayingStreams();
@@ -778,6 +761,12 @@ void setupStreams() {
 }
 
 void startPlayingStreams() {
+  if (duration == 0) {
+    if (scale > 0) duration = session->playEndTime() - initialSeekTime; // use SDP end time
+    else if (scale < 0) duration = initialSeekTime;
+  }
+  if (duration < 0) duration = 0.0;
+
   if (!clientStartPlayingSession(ourClient, session)) {
     *env << "Failed to start playing session: " << env->getResultMsg() << "\n";
     shutdown();
@@ -793,31 +782,21 @@ void startPlayingStreams() {
   // Figure out how long to delay (if at all) before shutting down, or
   // repeating the playing
   Boolean timerIsBeingUsed = False;
-  double totalEndTime = endTime;
-  if (endTime == 0) endTime = session->playEndTime(); // use SDP end time
-  if (endTime > 0) {
-    double const maxDelayTime
-      = (double)( ((unsigned)0x7FFFFFFF)/1000000.0 );
-    if (endTime > maxDelayTime) {
-      *env << "Warning: specified end time " << endTime
-		<< " exceeds maximum " << maxDelayTime
-		<< "; will not do a delayed shutdown\n";
-      endTime = 0.0;
-    } else {
-      timerIsBeingUsed = True;
-      totalEndTime = endTime + endTimeSlop;
+  double secondsToDelay = duration;
+  if (duration > 0) {
+    timerIsBeingUsed = True;
+    double absScale = scale > 0 ? scale : -scale; // ASSERT: scale != 0
+    secondsToDelay = duration/absScale + durationSlop;
 
-      int uSecsToDelay = (int)(totalEndTime*1000000.0);
-      sessionTimerTask = env->taskScheduler().scheduleDelayedTask(
-         uSecsToDelay, (TaskFunc*)sessionTimerHandler, (void*)NULL);
-    }
+    int64_t uSecsToDelay = (int64_t)(secondsToDelay*1000000.0);
+    sessionTimerTask = env->taskScheduler().scheduleDelayedTask(uSecsToDelay, (TaskFunc*)sessionTimerHandler, (void*)NULL);
   }
 
   char const* actionString
     = createReceivers? "Receiving streamed data":"Data is being streamed";
   if (timerIsBeingUsed) {
     *env << actionString
-		<< " (for up to " << totalEndTime
+		<< " (for up to " << secondsToDelay
 		<< " seconds)...\n";
   } else {
 #ifdef USE_SIGNALS
@@ -856,19 +835,19 @@ void closeMediaSinks() {
 }
 
 void subsessionAfterPlaying(void* clientData) {
-  // Begin by closing this media subsession:
+  // Begin by closing this media subsession's stream:
   MediaSubsession* subsession = (MediaSubsession*)clientData;
   Medium::close(subsession->sink);
   subsession->sink = NULL;
 
-  // Next, check whether *all* subsessions have now been closed:
+  // Next, check whether *all* subsessions' streams have now been closed:
   MediaSession& session = subsession->parentSession();
   MediaSubsessionIterator iter(session);
   while ((subsession = iter.next()) != NULL) {
     if (subsession->sink != NULL) return; // this subsession is still active
   }
 
-  // All subsessions have now been closed
+  // All subsessions' streams have now been closed
   sessionAfterPlaying();
 }
 
@@ -912,15 +891,6 @@ public:
       totNumPacketsReceived(0), totNumPacketsExpected(0) {
     measurementEndTime = measurementStartTime = startTime;
 
-#ifdef SUPPORT_REAL_RTSP
-    if (session->isRealNetworksRDT) { // hack for RealMedia sessions (RDT, not RTP)
-      RealRDTSource* rdt = (RealRDTSource*)src;
-      kBytesTotal = rdt->totNumKBytesReceived();
-      totNumPacketsReceived = rdt->totNumPacketsReceived();
-      totNumPacketsExpected = totNumPacketsReceived; // because we use TCP
-      return;
-    }
-#endif
     RTPReceptionStatsDB::Iterator statsIter(src->receptionStatsDB());
     // Assume that there's only one SSRC source (usually the case):
     RTPReceptionStats* stats = statsIter.next(True);
@@ -931,7 +901,7 @@ public:
     }
   }
   virtual ~qosMeasurementRecord() { delete fNext; }
-    
+
   void periodicQOSMeasurement(struct timeval const& timeNow);
 
 public:
@@ -957,8 +927,8 @@ static void scheduleNextQOSMeasurement() {
   struct timeval timeNow;
   gettimeofday(&timeNow, NULL);
   unsigned timeNowUSecs = timeNow.tv_sec*1000000 + timeNow.tv_usec;
-  unsigned usecsToDelay = nextQOSMeasurementUSecs < timeNowUSecs ? 0
-    : nextQOSMeasurementUSecs - timeNowUSecs;
+  unsigned usecsToDelay = nextQOSMeasurementUSecs - timeNowUSecs;
+     // Note: This works even when nextQOSMeasurementUSecs wraps around
 
   qosMeasurementTimerTask = env->taskScheduler().scheduleDelayedTask(
      usecsToDelay, (TaskFunc*)periodicQOSMeasurement, (void*)NULL);
@@ -984,24 +954,6 @@ void qosMeasurementRecord
   double timeDiff = secsDiff + usecsDiff/1000000.0;
   measurementEndTime = timeNow;
 
-#ifdef SUPPORT_REAL_RTSP
-  if (session->isRealNetworksRDT) { // hack for RealMedia sessions (RDT, not RTP)
-    RealRDTSource* rdt = (RealRDTSource*)fSource;
-    double kBytesTotalNow = rdt->totNumKBytesReceived();
-    double kBytesDeltaNow = kBytesTotalNow - kBytesTotal;
-    kBytesTotal = kBytesTotalNow;
-
-    double kbpsNow = timeDiff == 0.0 ? 0.0 : 8*kBytesDeltaNow/timeDiff;
-    if (kbpsNow < 0.0) kbpsNow = 0.0; // in case of roundoff error
-    if (kbpsNow < kbits_per_second_min) kbits_per_second_min = kbpsNow;
-    if (kbpsNow > kbits_per_second_max) kbits_per_second_max = kbpsNow;
-
-    totNumPacketsReceived = rdt->totNumPacketsReceived();
-    totNumPacketsExpected = totNumPacketsReceived; // because we use TCP
-    packet_loss_fraction_min = packet_loss_fraction_max = 0.0; // ditto
-    return;
-  }
-#endif
   RTPReceptionStatsDB::Iterator statsIter(fSource->receptionStatsDB());
   // Assume that there's only one SSRC source (usually the case):
   RTPReceptionStats* stats = statsIter.next(True);
@@ -1044,9 +996,6 @@ void beginQOSMeasurement() {
   MediaSubsession* subsession;
   while ((subsession = iter.next()) != NULL) {
     RTPSource* src = subsession->rtpSource();
-#ifdef SUPPORT_REAL_RTSP
-    if (session->isRealNetworksRDT) src = (RTPSource*)(subsession->readSource()); // hack
-#endif
     if (src == NULL) continue;
 
     qosMeasurementRecord* qosRecord
@@ -1061,11 +1010,8 @@ void beginQOSMeasurement() {
 }
 
 void printQOSData(int exitCode) {
-  if (exitCode != 0 && statusCode == 0) statusCode = 2;
   *env << "begin_QOS_statistics\n";
-  *env << "server_availability\t" << (statusCode == 1 ? 0 : 100) << "\n";
-  *env << "stream_availability\t" << (statusCode == 0 ? 100 : 0) << "\n";
-
+  
   // Print out stats for each active subsession:
   qosMeasurementRecord* curQOSRecord = qosRecordHead;
   if (session != NULL) {
@@ -1073,11 +1019,8 @@ void printQOSData(int exitCode) {
     MediaSubsession* subsession;
     while ((subsession = iter.next()) != NULL) {
       RTPSource* src = subsession->rtpSource();
-#ifdef SUPPORT_REAL_RTSP
-      if (session->isRealNetworksRDT) src = (RTPSource*)(subsession->readSource()); // hack
-#endif
       if (src == NULL) continue;
-
+      
       *env << "subsession\t" << subsession->mediumName()
 	   << "/" << subsession->codecName() << "\n";
       
@@ -1123,38 +1066,23 @@ void printQOSData(int exitCode) {
 	*env << "packet_loss_percentage_max\t"
 	     << (packetLossFraction == 1.0 ? 100.0 : 100*curQOSRecord->packet_loss_fraction_max) << "\n";
 	
-#ifdef SUPPORT_REAL_RTSP
-	if (session->isRealNetworksRDT) {
-	  RealRDTSource* rdt = (RealRDTSource*)src;
-	  *env << "inter_packet_gap_ms_min\t" << rdt->minInterPacketGapUS()/1000.0 << "\n";
-	  struct timeval totalGaps = rdt->totalInterPacketGaps();
+	RTPReceptionStatsDB::Iterator statsIter(src->receptionStatsDB());
+	// Assume that there's only one SSRC source (usually the case):
+	RTPReceptionStats* stats = statsIter.next(True);
+	if (stats != NULL) {
+	  *env << "inter_packet_gap_ms_min\t" << stats->minInterPacketGapUS()/1000.0 << "\n";
+	  struct timeval totalGaps = stats->totalInterPacketGaps();
 	  double totalGapsMS = totalGaps.tv_sec*1000.0 + totalGaps.tv_usec/1000.0;
-	  unsigned totNumPacketsReceived = rdt->totNumPacketsReceived();
+	  unsigned totNumPacketsReceived = stats->totNumPacketsReceived();
 	  *env << "inter_packet_gap_ms_ave\t"
 	       << (totNumPacketsReceived == 0 ? 0.0 : totalGapsMS/totNumPacketsReceived) << "\n";
-	  *env << "inter_packet_gap_ms_max\t" << rdt->maxInterPacketGapUS()/1000.0 << "\n";
-	} else {
-#endif
-	  RTPReceptionStatsDB::Iterator statsIter(src->receptionStatsDB());
-	  // Assume that there's only one SSRC source (usually the case):
-	  RTPReceptionStats* stats = statsIter.next(True);
-	  if (stats != NULL) {
-	    *env << "inter_packet_gap_ms_min\t" << stats->minInterPacketGapUS()/1000.0 << "\n";
-	    struct timeval totalGaps = stats->totalInterPacketGaps();
-	    double totalGapsMS = totalGaps.tv_sec*1000.0 + totalGaps.tv_usec/1000.0;
-	    unsigned totNumPacketsReceived = stats->totNumPacketsReceived();
-	    *env << "inter_packet_gap_ms_ave\t"
-		 << (totNumPacketsReceived == 0 ? 0.0 : totalGapsMS/totNumPacketsReceived) << "\n";
-	    *env << "inter_packet_gap_ms_max\t" << stats->maxInterPacketGapUS()/1000.0 << "\n";
-	  }
-#ifdef SUPPORT_REAL_RTSP
+	  *env << "inter_packet_gap_ms_max\t" << stats->maxInterPacketGapUS()/1000.0 << "\n";
 	}
-#endif
 	
 	curQOSRecord = curQOSRecord->fNext;
       }
     }
-  }    
+  }
 
   *env << "end_QOS_statistics\n";
   delete qosRecordHead;
@@ -1192,7 +1120,7 @@ void signalHandlerShutdown(int /*sig*/) {
 }
 
 void checkForPacketArrival(void* /*clientData*/) {
-  if (!notifyOnPacketArrival) return; // we're not checking 
+  if (!notifyOnPacketArrival) return; // we're not checking
 
   // Check each subsession, to see whether it has received data packets:
   unsigned numSubsessionsChecked = 0;
@@ -1236,7 +1164,7 @@ void checkForPacketArrival(void* /*clientData*/) {
     struct timeval timeNow;
     gettimeofday(&timeNow, NULL);
 	char timestampStr[100];
-	sprintf(timestampStr, "%ld%03ld", timeNow.tv_sec, timeNow.tv_usec/1000);
+	sprintf(timestampStr, "%ld%03ld", timeNow.tv_sec, (long)(timeNow.tv_usec/1000));
     *env << (syncStreams ? "Synchronized d" : "D")
 		<< "ata packets have begun arriving [" << timestampStr << "]\007\n";
     return;
@@ -1250,7 +1178,7 @@ void checkForPacketArrival(void* /*clientData*/) {
 }
 
 void checkInterPacketGaps(void* /*clientData*/) {
-  if (interPacketGapMaxTime == 0) return; // we're not checking 
+  if (interPacketGapMaxTime == 0) return; // we're not checking
 
   // Check each subsession, counting up how many packets have been received:
   unsigned newTotNumPacketsReceived = 0;
@@ -1271,7 +1199,7 @@ void checkInterPacketGaps(void* /*clientData*/) {
     sessionAfterPlaying();
   } else {
     totNumPacketsReceived = newTotNumPacketsReceived;
-    // Check again, after the specified delay: 
+    // Check again, after the specified delay:
     interPacketGapCheckTimerTask
       = env->taskScheduler().scheduleDelayedTask(interPacketGapMaxTime*1000000,
 				 (TaskFunc*)checkInterPacketGaps, NULL);
